/* This file is part of Zanshin

   Copyright 2014 Kevin Ottens <ervin@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
   USA.
*/


#ifndef WIDGETS_EDITORVIEW_H
#define WIDGETS_EDITORVIEW_H

#include <QWidget>

#include <QDateTime>

class QAbstractButton;
class QLabel;
class QLineEdit;
class QComboBox;
class QSpinBox;
class QVBoxLayout;
class QHBoxLayout;
class QListWidget;

class KLineEdit;
class KTimeComboBox;

namespace KPIM {
    class KDateEdit;
}

namespace Widgets {

class RecurrenceWidget;
class EditorWidget;

class EditorView : public QWidget
{
    Q_OBJECT
public:
    explicit EditorView(QWidget *parent = 0);
    virtual ~EditorView() {};

    QObject *model() const;

public slots:
    void setModel(QObject *model);

signals:
    void textChanged(const QString &text);
    void titleChanged(const QString &title);
    void startDateChanged(const QDateTime &start);
    void dueDateChanged(const QDateTime &due);
    void delegateChanged(const QString &name, const QString &email);
    void progressChanged(int progress);
    void statusChanged(int status);

    void alarmTypeChanged(int type);
    void alarmPositionChanged(int position);
    void alarmOffsetChanged(int offset);

private slots:
    void onArtifactChanged();
    void onHasTaskPropertiesChanged();
    void onTextOrTitleChanged();
    void onStartDateChanged();
    void onDueDateChanged();
    void onDelegateTextChanged();
    void onProgressChanged();
    void onStatusChanged();
    void onRelationsChanged();
    void onTagsChanged();
    void onAttachmentsChanged();
    void onRecurrenceChanged();

    void onTextEditChanged();
    void onStartEditEntered(const QDate &start);
    void onDueEditEntered(const QDate &due);
    void onStartTodayClicked();
    void onDelegateEntered();
    void onProgressChanged(int progress);
    void onStatusChanged(int status);
    void onTagAdded();

    void onLinkActivated(const QString &link);
    void onRemoveRelationClicked();
    void onAttachmentLinkActivated(const QString &link);
    void onRemoveAttachmentClicked();
    void onAddAttachmentClicked();
    void onStartTimeEntered(const QTime &start);
    void onDueTimeEntered(const QTime &due);
    void onRemoveTagClicked();

    void onAlarmChanged();
    void onAlarmTypeChanged(int type);
    void onAlarmPositionChanged(int);
    void onAlarmOffsetChanged(int);
    void onAlarmEnableChanged(int);

    void toggleFullscreenEditor();

private:
    void fillAlarmCombo();

    QObject *m_model;

    QLabel *m_delegateLabel;
    QLineEdit *m_titleEdit;
    EditorWidget *m_textEdit;
    QWidget *m_taskGroup;
    KPIM::KDateEdit *m_startDateEdit;
    KTimeComboBox *m_startTimeEdit;
    KPIM::KDateEdit *m_dueDateEdit;
    KTimeComboBox *m_dueTimeEdit;
    QAbstractButton *m_startTodayButton;
    KLineEdit *m_delegateEdit;
    QComboBox *m_statusComboBox;
    QSpinBox *m_progressEdit;
    QList<QWidget*> m_relationWidgets;
    QVBoxLayout *m_relationsLayout;
    QList<QWidget*> m_attachmentWidgets;
    QVBoxLayout *m_attachmentsLayout;
    QList<QWidget*> m_tagsWidgets;
    QHBoxLayout *m_tagsLayout;
    QAbstractButton *m_addAttachmentButton;
    RecurrenceWidget *m_recurrenceWidget;
    QComboBox *m_alarmTypeWidget;
    QComboBox *m_alarmPositionWidget;
    QSpinBox *m_alarmOffsetEdit;
    QLineEdit *m_newTagWidget;
};

}

#endif // WIDGETS_EDITORVIEW_H
